( cd usr/lib ; rm -rf libgnome-menu.so )
( cd usr/lib ; ln -sf libgnome-menu.so.2.1.19 libgnome-menu.so )
( cd usr/lib ; rm -rf libgnome-menu.so.2 )
( cd usr/lib ; ln -sf libgnome-menu.so.2.1.19 libgnome-menu.so.2 )
